@@ -24,7 +24,7 @@ from typing import (
 import tomli
 from pkg_resources import parse_version
 
-from nbqa import replace_source, save_source
+from nbqa import replace_source, save_code_source, save_markdown_source
 from nbqa.cmdline import CLIArgs
 from nbqa.config.config import Configs, get_default_config
 from nbqa.find_root import find_project_root
@@ -32,7 +32,7 @@ from nbqa.notebook_info import NotebookInfo
 from nbqa.optional import metadata
 from nbqa.output_parser import Output, map_python_line_to_nb_lines
 from nbqa.path_utils import get_relative_and_absolute_paths, remove_suffix
-from nbqa.save_source import CODE_SEPARATOR
+from nbqa.save_code_source import CODE_SEPARATOR
 from nbqa.text import BOLD, RESET
 
 BASE_ERROR_MESSAGE = (
@@ -53,6 +53,7 @@ REPLACE_FUNCTION = {
     True: replace_source.diff,
     False: replace_source.mutate,
 }
+SUFFIX = {False: ".py", True: ".md"}
 
 
 class TemporaryFile(NamedTuple):
@@ -62,6 +63,14 @@ class TemporaryFile(NamedTuple):
     file: str
 
 
+class SavedSources(NamedTuple):
+    """Mapping between notebooks and Python files, failed notebooks, non-Python notebooks."""
+
+    nb_info_mapping: Mapping[str, NotebookInfo]
+    failed_notebooks: MutableMapping[str, str]
+    non_python_notebooks: Set[str]
+
+
 class UnsupportedPackageVersionError(Exception):
     """Raise if installed module is older than minimum required version."""
 
@@ -158,6 +167,8 @@ def _replace_temp_python_file_references_in_out_err(
     notebook: str,
     out: str,
     err: str,
+    *,
+    md: bool,
 ) -> Output:
     """
     Replace references to temporary Python file with references to notebook.
@@ -184,10 +195,10 @@ def _replace_temp_python_file_references_in_out_err(
     err = err.replace(py_basename, nb_basename)
 
     out = out.replace(
-        remove_suffix(py_basename, ".py"), remove_suffix(nb_basename, ".ipynb")
+        remove_suffix(py_basename, SUFFIX[md]), remove_suffix(nb_basename, ".ipynb")
     )
     err = err.replace(
-        remove_suffix(py_basename, ".py"), remove_suffix(nb_basename, ".ipynb")
+        remove_suffix(py_basename, SUFFIX[md]), remove_suffix(nb_basename, ".ipynb")
     )
 
     return Output(out, err)
@@ -243,11 +254,16 @@ def _run_command(
     """
     before = [_get_mtimes(i) for i in args]
 
+    my_env = os.environ.copy()
+    if command == "mypy" and "MYPY_FORCE_COLOR" not in my_env:
+        my_env["MYPY_FORCE_COLOR"] = "1"
+
     output = subprocess.run(
         [sys.executable, "-m", command, *args, *cmd_args],
         stderr=subprocess.PIPE,
         stdout=subprocess.PIPE,
         universal_newlines=True,  # from Python3.7 this can be replaced with `text`
+        env=my_env,
     )
 
     mutated = [_get_mtimes(i) for i in args] != before
@@ -354,11 +370,11 @@ def _clean_up_tmp_files(nb_to_py_mapping: Mapping[str, Tuple[int, str]]) -> None
         os.remove(tmp_path)
 
 
-def _get_nb_to_py_mapping(
-    root_dirs: Sequence[str], files: Optional[str], exclude: Optional[str]
+def _get_nb_to_tmp_mapping(
+    root_dirs: Sequence[str], files: Optional[str], exclude: Optional[str], md: bool
 ) -> Dict[str, TemporaryFile]:
     """
-    Get mapping between notebooks and temporary Python files.
+    Get mapping between notebooks and temporary files.
 
     Parameters
     ----------
@@ -372,35 +388,35 @@ def _get_nb_to_py_mapping(
     Returns
     -------
     Dict[str, Tuple[int, str]]
-        Mapping between notebooks and temporary Python files.
+        Mapping between notebooks and temporary files.
 
     Raises
     ------
     FileNotFoundError
         If notebook isn't found.
     """
-    nb_to_py_mapping: Dict[str, TemporaryFile] = {}
+    nb_to_tmp_mapping: Dict[str, TemporaryFile] = {}
     for notebook in _get_all_notebooks(root_dirs, files, exclude):
         if not os.path.exists(notebook):
-            _clean_up_tmp_files(nb_to_py_mapping)
+            _clean_up_tmp_files(nb_to_tmp_mapping)
             raise FileNotFoundError(
                 f"{BOLD}No such file or directory: {notebook}{RESET}\n"
             )
 
-        nb_to_py_mapping[notebook] = TemporaryFile(
+        nb_to_tmp_mapping[notebook] = TemporaryFile(
             *tempfile.mkstemp(
                 dir=os.path.dirname(notebook),
                 prefix=remove_suffix(os.path.basename(notebook), ".ipynb"),
-                suffix=".py",
+                suffix=SUFFIX[md],
             )
         )
         relative_path, _ = get_relative_and_absolute_paths(
-            nb_to_py_mapping[notebook].file
+            nb_to_tmp_mapping[notebook].file
         )
-        nb_to_py_mapping[notebook] = nb_to_py_mapping[notebook]._replace(
+        nb_to_tmp_mapping[notebook] = nb_to_tmp_mapping[notebook]._replace(
             file=relative_path
         )
-    return nb_to_py_mapping
+    return nb_to_tmp_mapping
 
 
 def _print_failed_notebook_errors(failed_notebooks: Mapping[str, str]) -> None:
@@ -428,9 +444,122 @@ def _is_non_python_notebook(notebook: MutableMapping[str, Any]) -> bool:
     return language is not None and language != "python"
 
 
-def _main(  # pylint: disable=R0912,R0914,R0911
-    cli_args: CLIArgs, configs: Configs
-) -> int:
+def _save_code_sources(
+    nb_to_py_mapping: Dict[str, TemporaryFile],
+    process_cells: Sequence[str],
+    skip_celltags: Sequence[str],
+    dont_skip_bad_cells: bool,
+    command: str,
+) -> SavedSources:
+    """
+    Save sources of notebooks.
+
+    Record which notebooks fail to process, and which ones are non-Python ones.
+    """
+    failed_notebooks = {}
+    non_python_notebooks = set()
+    nb_info_mapping: MutableMapping[str, NotebookInfo] = {}
+
+    for notebook, (file_descriptor, _) in nb_to_py_mapping.items():
+        with open(str(notebook), encoding="utf-8") as handle:
+            content = handle.read()
+        try:
+            notebook_json = json.loads(content)
+            if _is_non_python_notebook(notebook_json):
+                non_python_notebooks.add(notebook)
+                continue
+            nb_info_mapping[notebook] = save_code_source.main(
+                notebook_json,
+                file_descriptor,
+                process_cells,
+                command,
+                skip_celltags,
+                dont_skip_bad_cells=dont_skip_bad_cells,
+            )
+        except Exception as exp_repr:  # pylint: disable=W0703
+            failed_notebooks[notebook] = repr(exp_repr)
+    return SavedSources(nb_info_mapping, failed_notebooks, non_python_notebooks)
+
+
+def _save_markdown_sources(
+    nb_to_md_mapping: Dict[str, TemporaryFile],
+    process_cells: Sequence[str],  # pylint: disable=W0613
+    skip_celltags: Sequence[str],
+    dont_skip_bad_cells: bool,  # pylint: disable=W0613
+    command: str,  # pylint: disable=W0613
+) -> SavedSources:
+    """
+    Save markdown sources of notebooks.
+
+    Record which notebooks fail to process.
+    """
+    failed_notebooks = {}
+    nb_info_mapping: MutableMapping[str, NotebookInfo] = {}
+
+    for notebook, (file_descriptor, _) in nb_to_md_mapping.items():
+        with open(str(notebook), encoding="utf-8") as handle:
+            content = handle.read()
+        try:
+            notebook_json = json.loads(content)
+            nb_info_mapping[notebook] = save_markdown_source.main(
+                notebook_json,
+                file_descriptor,
+                skip_celltags,
+            )
+        except Exception as exp_repr:  # pylint: disable=W0703
+            failed_notebooks[notebook] = repr(exp_repr)
+    return SavedSources(nb_info_mapping, failed_notebooks, set())
+
+
+SAVE_SOURCES = {False: _save_code_sources, True: _save_markdown_sources}
+
+
+def _post_process_notebooks(  # pylint: disable=R0913
+    saved_sources: SavedSources,
+    nb_to_py_mapping: Mapping[str, TemporaryFile],
+    mutated: bool,
+    diff: bool,
+    command: str,
+    output: Output,
+    *,
+    md: bool,
+) -> Tuple[bool, Output]:
+    """Replace source in notebooks, modify output so it refers to notebooks."""
+    actually_mutated = False
+    for notebook, (_, temp_python_file) in nb_to_py_mapping.items():
+        if (
+            notebook in saved_sources.failed_notebooks
+            or notebook in saved_sources.non_python_notebooks
+        ):
+            continue
+        output = _replace_temp_python_file_references_in_out_err(
+            temp_python_file, notebook, output.out, output.err, md=md
+        )
+        output = map_python_line_to_nb_lines(
+            command,
+            output.out,
+            output.err,
+            notebook,
+            saved_sources.nb_info_mapping[notebook].cell_mappings,
+        )
+
+        if mutated:
+            try:
+                actually_mutated = (
+                    REPLACE_FUNCTION[diff](
+                        temp_python_file,
+                        notebook,
+                        saved_sources.nb_info_mapping[notebook],
+                        md=md,
+                    )
+                    or actually_mutated
+                )
+            except Exception as exp_repr:  # pylint: disable=W0703
+                saved_sources.failed_notebooks[notebook] = repr(exp_repr)
+    return actually_mutated, output
+
+
+def _main(cli_args: CLIArgs, configs: Configs) -> int:
     """
     Run third-party tool on a single notebook or directory.
 
@@ -447,48 +576,32 @@ def _main(  # pylint: disable=R0912,R0914,R0911
         Output code from third-party tool.
     """
     try:
-        nb_to_py_mapping = _get_nb_to_py_mapping(
-            cli_args.root_dirs, configs["files"], configs["exclude"]
+        nb_to_tmp_mapping = _get_nb_to_tmp_mapping(
+            cli_args.root_dirs, configs["files"], configs["exclude"], configs["md"]
         )
     except FileNotFoundError as exc:
         sys.stderr.write(str(exc))
         return 1
 
-    failed_notebooks = {}
-    non_python_notebooks = set()
     try:  # pylint disable=R0912
 
-        if not nb_to_py_mapping:
+        if not nb_to_tmp_mapping:
             sys.stderr.write(
                 "No .ipynb notebooks found in given directories: "
                 f"{' '.join(i for i in cli_args.root_dirs if os.path.isdir(i))}\n"
             )
             return 0
+        saved_sources = SAVE_SOURCES[configs["md"]](
+            nb_to_tmp_mapping,
+            configs["process_cells"],
+            configs["skip_celltags"],
+            configs["dont_skip_bad_cells"],
+            cli_args.command,
+        )
 
-        nb_info_mapping: MutableMapping[str, NotebookInfo] = {}
-
-        for notebook, (file_descriptor, _) in nb_to_py_mapping.items():
-            with open(str(notebook), encoding="utf-8") as handle:
-                content = handle.read()
-            try:
-                notebook_json = json.loads(content)
-                if _is_non_python_notebook(notebook_json):
-                    non_python_notebooks.add(notebook)
-                    continue
-                nb_info_mapping[notebook] = save_source.main(
-                    notebook_json,
-                    file_descriptor,
-                    configs["process_cells"],
-                    cli_args.command,
-                    configs["skip_celltags"],
-                    dont_skip_bad_cells=configs["dont_skip_bad_cells"],
-                )
-            except Exception as exp_repr:  # pylint: disable=W0703
-                failed_notebooks[notebook] = repr(exp_repr)
-
-        if len(failed_notebooks) == len(nb_to_py_mapping):
+        if len(saved_sources.failed_notebooks) == len(nb_to_tmp_mapping):
             sys.stderr.write("No valid .ipynb notebooks found\n")
-            _print_failed_notebook_errors(failed_notebooks)
+            _print_failed_notebook_errors(saved_sources.failed_notebooks)
             return 123
 
         output, output_code, mutated = _run_command(
@@ -496,38 +609,20 @@ def _main(  # pylint: disable=R0912,R0914,R0911
             configs["addopts"],
             [
                 i.file
-                for key, i in nb_to_py_mapping.items()
-                if key not in failed_notebooks
+                for key, i in nb_to_tmp_mapping.items()
+                if key not in saved_sources.failed_notebooks
             ],
         )
 
-        actually_mutated = False
-        for notebook, (_, temp_python_file) in nb_to_py_mapping.items():
-            if notebook in failed_notebooks or notebook in non_python_notebooks:
-                continue
-            output = _replace_temp_python_file_references_in_out_err(
-                temp_python_file, notebook, output.out, output.err
-            )
-            output = map_python_line_to_nb_lines(
-                cli_args.command,
-                output.out,
-                output.err,
-                notebook,
-                nb_info_mapping[notebook].cell_mappings,
-            )
-
-            if mutated:
-                try:
-                    actually_mutated = (
-                        REPLACE_FUNCTION[configs["diff"]](
-                            temp_python_file,
-                            notebook,
-                            nb_info_mapping[notebook],
-                        )
-                        or actually_mutated
-                    )
-                except Exception as exp_repr:  # pylint: disable=W0703
-                    failed_notebooks[notebook] = repr(exp_repr)
+        actually_mutated, output = _post_process_notebooks(
+            saved_sources,
+            nb_to_tmp_mapping,
+            mutated,
+            configs["diff"],
+            cli_args.command,
+            output,
+            md=configs["md"],
+        )
 
         sys.stdout.write(output.out)
         sys.stderr.write(output.err)
@@ -536,9 +631,9 @@ def _main(  # pylint: disable=R0912,R0914,R0911
             output_code = 0
             mutated = False
 
-        if failed_notebooks:
+        if saved_sources.failed_notebooks:
             output_code = 123
-            _print_failed_notebook_errors(failed_notebooks)
+            _print_failed_notebook_errors(saved_sources.failed_notebooks)
 
         if configs["diff"]:
             if mutated:
@@ -548,7 +643,7 @@ def _main(  # pylint: disable=R0912,R0914,R0911
             return output_code
 
     finally:
-        _clean_up_tmp_files(nb_to_py_mapping)
+        _clean_up_tmp_files(nb_to_tmp_mapping)
     return output_code
 
 
@@ -575,9 +670,9 @@ def _check_command_is_installed(command: str) -> None:
             import_module(command)
         except ImportError as exc:
             if not os.path.isdir(command) and not os.path.isfile(
-                f"{command}.py"
+                f"{os.path.join(*command.split('.'))}.py"
             ):  # pragma: nocover(py<37)
-                # I presume there lack of coverage in Python3.6 here is a bug, as all
+                # I presume the lack of coverage in Python3.6 here is a bug, as all
                 # these branches are actually covered.
                 raise ModuleNotFoundError(_get_command_not_found_msg(command)) from exc
     else:
